##############################################################################
#
# Copyright (c) 2005 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for GenericSetup export / import support for topics / criteria.

$Id: test_exportimport.py 71240 2006-11-21 13:14:15Z yuppie $
"""

import unittest

from DateTime.DateTime import DateTime

from Products.GenericSetup.testing import ExportImportZCMLLayer
from Products.GenericSetup.tests.conformance \
        import ConformsToIFilesystemExporter
from Products.GenericSetup.tests.conformance \
        import ConformsToIFilesystemImporter
from Products.GenericSetup.tests.common import BaseRegistryTests
from Products.GenericSetup.tests.common import DummyExportContext
from Products.GenericSetup.tests.common import DummyImportContext

_DATE_STR = '2005-11-20T12:00:00Z'
_CRITERIA_DATA = (
    ('a', 'String Criterion', {'value': 'A'}),
    ('b', 'Integer Criterion', {'value': 3, 'direction': 'min'}),
    ('c', 'Friendly Date Criterion', {'value': DateTime(_DATE_STR),
                                      'operation': 'min',
                                      'daterange': 'old',
                                     }),
    ('d', 'List Criterion', {'value': ('D', 'd'), 'operator': 'or'}),
    ('e', 'Sort Criterion', {'reversed': 0}),
)


class TopicExportImportTests(BaseRegistryTests,
                             ConformsToIFilesystemExporter,
                             ConformsToIFilesystemImporter,
                            ):

    layer = ExportImportZCMLLayer

    def _getTargetClass(self):
        from Products.CMFTopic.exportimport import TopicExportImport
        return TopicExportImport

    def _makeOne(self, context, *args, **kw):
        return self._getTargetClass()(context, *args, **kw)

    def _makeTopic(self, id, with_criteria=False):
        from Products.CMFTopic.Topic import Topic
        topic = Topic(id)

        if with_criteria:
            for field, c_type, attrs in _CRITERIA_DATA:
                topic.addCriterion(field, c_type)
                criterion = topic.getCriterion(field)
                criterion.edit(**attrs)

        return topic

    def test_listExportableItems(self):
        topic = self._makeTopic('lEI', False).__of__(self.root)
        adapter = self._makeOne(topic)

        self.assertEqual(len(adapter.listExportableItems()), 0)
        topic.addCriterion('field_a', 'String Criterion')
        self.assertEqual(len(adapter.listExportableItems()), 0)

    def test__getExportInfo_empty(self):
        topic = self._makeTopic('empty', False).__of__(self.root)
        adapter = self._makeOne(topic)

        info = adapter._getExportInfo()
        self.assertEqual(len(info['criteria']), 0)

    def test_export_empty(self):
        topic = self._makeTopic('empty', False).__of__(self.root)
        adapter = self._makeOne(topic)

        context = DummyExportContext(topic)
        adapter.export(context, 'test', False)

        self.assertEqual( len( context._wrote ), 2 )
        filename, text, content_type = context._wrote[ 0 ]
        self.assertEqual( filename, 'test/empty/.objects' )
        self.assertEqual( text, '' )
        self.assertEqual( content_type, 'text/comma-separated-values' )

        filename, text, content_type = context._wrote[ 1 ]
        self.assertEqual( filename, 'test/empty/criteria.xml' )
        self._compareDOM( text, _EMPTY_TOPIC_CRITERIA )
        self.assertEqual( content_type, 'text/xml' )

    def test__getExportInfo_with_criteria(self):
        topic = self._makeTopic('with_criteria', True).__of__(self.root)
        adapter = self._makeOne(topic)

        info = adapter._getExportInfo()
        self.assertEqual(len(info['criteria']), len(_CRITERIA_DATA))

        for found, expected in zip(info['criteria'], _CRITERIA_DATA):
            attributes = expected[2]
            for k, v in attributes.items():
                if type(v) in (list, tuple):
                    attributes[k] = ','.join(v)

            self.assertEqual(found['criterion_id'], 'crit__%s' % expected[0])
            self.assertEqual(found['type'], expected[1])
            self.assertEqual(found['field'], expected[0])
            self.assertEqual(dict(found['attributes']), attributes)

    def test_export_with_string_criterion(self):
        topic = self._makeTopic('with_string', False).__of__(self.root)
        data = _CRITERIA_DATA[0]
        topic.addCriterion(data[0], data[1])
        topic.getCriterion(data[0]).edit(**data[2])
        adapter = self._makeOne(topic)

        context = DummyExportContext(topic)
        adapter.export(context, 'test', False)

        self.assertEqual( len( context._wrote ), 2 )
        filename, text, content_type = context._wrote[ 0 ]
        self.assertEqual( filename, 'test/with_string/.objects' )
        self.assertEqual( text, '' )
        self.assertEqual( content_type, 'text/comma-separated-values' )

        filename, text, content_type = context._wrote[ 1 ]
        self.assertEqual( filename, 'test/with_string/criteria.xml' )
        self._compareDOM( text, _STRING_TOPIC_CRITERIA )
        self.assertEqual( content_type, 'text/xml' )

    def test_export_with_integer_criterion(self):
        topic = self._makeTopic('with_integer', False).__of__(self.root)
        data = _CRITERIA_DATA[1]
        topic.addCriterion(data[0], data[1])
        topic.getCriterion(data[0]).edit(**data[2])
        adapter = self._makeOne(topic)

        context = DummyExportContext(topic)
        adapter.export(context, 'test', False)

        self.assertEqual( len( context._wrote ), 2 )
        filename, text, content_type = context._wrote[ 0 ]
        self.assertEqual( filename, 'test/with_integer/.objects' )
        self.assertEqual( text, '' )
        self.assertEqual( content_type, 'text/comma-separated-values' )

        filename, text, content_type = context._wrote[ 1 ]
        self.assertEqual( filename, 'test/with_integer/criteria.xml' )
        self._compareDOM( text, _INTEGER_TOPIC_CRITERIA )
        self.assertEqual( content_type, 'text/xml' )

    def test_export_with_date_criterion(self):
        topic = self._makeTopic('with_date', False).__of__(self.root)
        data = _CRITERIA_DATA[2]
        topic.addCriterion(data[0], data[1])
        topic.getCriterion(data[0]).edit(**data[2])
        adapter = self._makeOne(topic)

        context = DummyExportContext(topic)
        adapter.export(context, 'test', False)

        self.assertEqual( len( context._wrote ), 2 )
        filename, text, content_type = context._wrote[ 0 ]
        self.assertEqual( filename, 'test/with_date/.objects' )
        self.assertEqual( text, '' )
        self.assertEqual( content_type, 'text/comma-separated-values' )

        filename, text, content_type = context._wrote[ 1 ]
        self.assertEqual( filename, 'test/with_date/criteria.xml' )
        self._compareDOM( text, _DATE_TOPIC_CRITERIA )
        self.assertEqual( content_type, 'text/xml' )

    def test_export_with_list_criterion(self):
        topic = self._makeTopic('with_list', False).__of__(self.root)
        data = _CRITERIA_DATA[3]
        topic.addCriterion(data[0], data[1])
        topic.getCriterion(data[0]).edit(**data[2])
        adapter = self._makeOne(topic)

        context = DummyExportContext(topic)
        adapter.export(context, 'test', False)

        self.assertEqual( len( context._wrote ), 2 )
        filename, text, content_type = context._wrote[ 0 ]
        self.assertEqual( filename, 'test/with_list/.objects' )
        self.assertEqual( text, '' )
        self.assertEqual( content_type, 'text/comma-separated-values' )

        filename, text, content_type = context._wrote[ 1 ]
        self.assertEqual( filename, 'test/with_list/criteria.xml' )
        self._compareDOM( text, _LIST_TOPIC_CRITERIA )
        self.assertEqual( content_type, 'text/xml' )

    def test_export_with_sort_criterion(self):
        topic = self._makeTopic('with_sort', False).__of__(self.root)
        data = _CRITERIA_DATA[4]
        topic.addCriterion(data[0], data[1])
        topic.getCriterion(data[0]).edit(**data[2])
        adapter = self._makeOne(topic)

        context = DummyExportContext(topic)
        adapter.export(context, 'test', False)

        self.assertEqual( len( context._wrote ), 2 )
        filename, text, content_type = context._wrote[ 0 ]
        self.assertEqual( filename, 'test/with_sort/.objects' )
        self.assertEqual( text, '' )
        self.assertEqual( content_type, 'text/comma-separated-values' )

        filename, text, content_type = context._wrote[ 1 ]
        self.assertEqual( filename, 'test/with_sort/criteria.xml' )
        self._compareDOM( text, _SORT_TOPIC_CRITERIA )
        self.assertEqual( content_type, 'text/xml' )

    def test_export_with_mixed_criteria(self):
        topic = self._makeTopic('with_mixed', False).__of__(self.root)
        for index in 0, 2, 4:
            data = _CRITERIA_DATA[index]
            topic.addCriterion(data[0], data[1])
            topic.getCriterion(data[0]).edit(**data[2])
        adapter = self._makeOne(topic)

        context = DummyExportContext(topic)
        adapter.export(context, 'test', False)

        self.assertEqual( len( context._wrote ), 2 )
        filename, text, content_type = context._wrote[ 0 ]
        self.assertEqual( filename, 'test/with_mixed/.objects' )
        self.assertEqual( text, '' )
        self.assertEqual( content_type, 'text/comma-separated-values' )

        filename, text, content_type = context._wrote[ 1 ]
        self.assertEqual( filename, 'test/with_mixed/criteria.xml' )
        self._compareDOM( text, _MIXED_TOPIC_CRITERIA )
        self.assertEqual( content_type, 'text/xml' )

    def test_import_empty_with_string_criterion(self):
        topic = self._makeTopic('empty', False).__of__(self.root)
        adapter = self._makeOne(topic)

        context = DummyImportContext(topic, encoding='ascii')
        context._files['test/empty/criteria.xml'] = _STRING_TOPIC_CRITERIA

        adapter.import_(context, 'test', False)

        expected = _CRITERIA_DATA[0]
        found = topic.listCriteria()
        self.assertEqual(len(found), 1)

        criterion = found[0]

        self.assertEqual(criterion.getId(), 'crit__%s' % expected[0])
        self.assertEqual(criterion.Type(), expected[1])
        self.assertEqual(criterion.Field(), expected[0])
        self.assertEqual(criterion.value, expected[2]['value'])

    def test_import_empty_with_integer_criterion(self):
        topic = self._makeTopic('empty', False).__of__(self.root)
        adapter = self._makeOne(topic)

        context = DummyImportContext(topic, encoding='ascii')
        context._files['test/empty/criteria.xml'] = _INTEGER_TOPIC_CRITERIA

        adapter.import_(context, 'test', False)

        expected = _CRITERIA_DATA[1]
        found = topic.listCriteria()
        self.assertEqual(len(found), 1)

        criterion = found[0]

        self.assertEqual(criterion.getId(), 'crit__%s' % expected[0])
        self.assertEqual(criterion.Type(), expected[1])
        self.assertEqual(criterion.Field(), expected[0])
        self.assertEqual(criterion.value, expected[2]['value'])
        self.assertEqual(criterion.direction, expected[2]['direction'])

    def test_import_empty_with_date_criterion(self):
        topic = self._makeTopic('empty', False).__of__(self.root)
        adapter = self._makeOne(topic)

        context = DummyImportContext(topic, encoding='ascii')
        context._files['test/empty/criteria.xml'] = _DATE_TOPIC_CRITERIA

        adapter.import_(context, 'test', False)

        expected = _CRITERIA_DATA[2]
        found = topic.listCriteria()
        self.assertEqual(len(found), 1)

        criterion = found[0]

        self.assertEqual(criterion.getId(), 'crit__%s' % expected[0])
        self.assertEqual(criterion.Type(), expected[1])
        self.assertEqual(criterion.Field(), expected[0])
        self.assertEqual(criterion.value, expected[2]['value'])
        self.assertEqual(criterion.operation, expected[2]['operation'])
        self.assertEqual(criterion.daterange, expected[2]['daterange'])

    def test_import_empty_with_list_criterion(self):
        topic = self._makeTopic('empty', False).__of__(self.root)
        adapter = self._makeOne(topic)

        context = DummyImportContext(topic, encoding='ascii')
        context._files['test/empty/criteria.xml'] = _LIST_TOPIC_CRITERIA

        adapter.import_(context, 'test', False)

        expected = _CRITERIA_DATA[3]
        found = topic.listCriteria()
        self.assertEqual(len(found), 1)

        criterion = found[0]

        self.assertEqual(criterion.getId(), 'crit__%s' % expected[0])
        self.assertEqual(criterion.Type(), expected[1])
        self.assertEqual(criterion.Field(), expected[0])
        self.assertEqual(','.join(criterion.value), expected[2]['value'])
        self.assertEqual(criterion.operator, expected[2]['operator'])

    def test_import_empty_with_sort_criterion(self):
        topic = self._makeTopic('empty', False).__of__(self.root)
        adapter = self._makeOne(topic)

        context = DummyImportContext(topic, encoding='ascii')
        context._files['test/empty/criteria.xml'] = _SORT_TOPIC_CRITERIA

        adapter.import_(context, 'test', False)

        expected = _CRITERIA_DATA[4]
        found = topic.listCriteria()
        self.assertEqual(len(found), 1)

        criterion = found[0]

        self.assertEqual(criterion.getId(), 'crit__%s' % expected[0])
        self.assertEqual(criterion.Type(), expected[1])
        self.assertEqual(criterion.field, None)
        self.assertEqual(criterion.index, expected[0])
        self.assertEqual(criterion.reversed, bool(expected[2]['reversed']))

    def test_import_empty_with_mixed_criterion(self):
        topic = self._makeTopic('empty', False).__of__(self.root)
        adapter = self._makeOne(topic)

        context = DummyImportContext(topic, encoding='ascii')
        context._files['test/empty/criteria.xml'] = _MIXED_TOPIC_CRITERIA

        adapter.import_(context, 'test', False)

        found = topic.listCriteria()
        self.assertEqual(len(found), 3)

        criterion = found[0]
        expected = _CRITERIA_DATA[0]

        self.assertEqual(criterion.getId(), 'crit__%s' % expected[0])
        self.assertEqual(criterion.Type(), expected[1])
        self.assertEqual(criterion.Field(), expected[0])
        self.assertEqual(criterion.value, expected[2]['value'])

        criterion = found[1]
        expected = _CRITERIA_DATA[2]

        self.assertEqual(criterion.getId(), 'crit__%s' % expected[0])
        self.assertEqual(criterion.Type(), expected[1])
        self.assertEqual(criterion.Field(), expected[0])
        self.assertEqual(criterion.value, expected[2]['value'])
        self.assertEqual(criterion.operation, expected[2]['operation'])
        self.assertEqual(criterion.daterange, expected[2]['daterange'])

        criterion = found[2]
        expected = _CRITERIA_DATA[4]

        self.assertEqual(criterion.getId(), 'crit__%s' % expected[0])
        self.assertEqual(criterion.Type(), expected[1])
        self.assertEqual(criterion.field, None)
        self.assertEqual(criterion.index, expected[0])
        self.assertEqual(criterion.reversed, bool(expected[2]['reversed']))

    def test_import_without_purge_leaves_existing_criteria(self):

        topic = self._makeTopic('with_criteria', True).__of__(self.root)
        adapter = self._makeOne(topic)

        context = DummyImportContext(topic, purge=False)
        context._files['test/with_criteria/criteria.xml'
                      ] = _EMPTY_TOPIC_CRITERIA

        self.assertEqual(len(topic.listCriteria()), len(_CRITERIA_DATA))
        adapter.import_(context, 'test', False)
        self.assertEqual(len(topic.listCriteria()), len(_CRITERIA_DATA))

    def test_import_with_purge_removes_existing_criteria(self):

        topic = self._makeTopic('with_criteria', True).__of__(self.root)
        adapter = self._makeOne(topic)

        context = DummyImportContext(topic, purge=True)
        context._files['test/with_criteria/criteria.xml'
                      ] = _EMPTY_TOPIC_CRITERIA

        self.assertEqual(len(topic.listCriteria()), len(_CRITERIA_DATA))
        adapter.import_(context, 'test', False)
        self.assertEqual(len(topic.listCriteria()), 0)

_EMPTY_TOPIC_CRITERIA = """\
<?xml version="1.0" ?>
<criteria>
</criteria>
"""

_STRING_TOPIC_CRITERIA = """\
<?xml version="1.0" ?>
<criteria>
 <criterion
    criterion_id="crit__a"
    type="String Criterion"
    field="a">
  <attribute name="value" value="A" />
 </criterion>
</criteria>
"""

_INTEGER_TOPIC_CRITERIA = """\
<?xml version="1.0" ?>
<criteria>
 <criterion
    criterion_id="crit__b"
    type="Integer Criterion"
    field="b">
  <attribute name="value" value="3" />
  <attribute name="direction" value="min" />
 </criterion>
</criteria>
"""

_DATE_TOPIC_CRITERIA = """\
<?xml version="1.0" ?>
<criteria>
 <criterion
    criterion_id="crit__c"
    type="Friendly Date Criterion"
    field="c">
  <attribute name="value" value="%s" />
  <attribute name="operation" value="min" />
  <attribute name="daterange" value="old" />
 </criterion>
</criteria>
""" % int(DateTime(_DATE_STR))

_LIST_TOPIC_CRITERIA = """\
<?xml version="1.0" ?>
<criteria>
 <criterion
    criterion_id="crit__d"
    type="List Criterion"
    field="d">
  <attribute name="value" value="D,d" />
  <attribute name="operator" value="or" />
 </criterion>
</criteria>
"""

_SORT_TOPIC_CRITERIA = """\
<?xml version="1.0" ?>
<criteria>
 <criterion
    criterion_id="crit__e"
    type="Sort Criterion"
    field="e">
  <attribute name="reversed" value="False" />
 </criterion>
</criteria>
"""

_MIXED_TOPIC_CRITERIA = """\
<?xml version="1.0" ?>
<criteria>
 <criterion
    criterion_id="crit__a"
    type="String Criterion"
    field="a">
  <attribute name="value" value="A" />
 </criterion>
 <criterion
    criterion_id="crit__c"
    type="Friendly Date Criterion"
    field="c">
  <attribute name="value" value="%s" />
  <attribute name="operation" value="min" />
  <attribute name="daterange" value="old" />
 </criterion>
 <criterion
    criterion_id="crit__e"
    type="Sort Criterion"
    field="e">
  <attribute name="reversed" value="False" />
 </criterion>
</criteria>
""" % int(DateTime(_DATE_STR))


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(TopicExportImportTests),
        ))

if __name__ == '__main__':
    from Products.GenericSetup.testing import run
    run(test_suite())



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Simple string-matching criterion class

$Id: SimpleStringCriterion.py 94489 2009-01-03 16:28:19Z yuppie $
"""

from AccessControl.SecurityInfo import ClassSecurityInfo
from App.class_init import InitializeClass
from zope.interface import implements

from Products.CMFTopic.AbstractCriterion import AbstractCriterion
from Products.CMFTopic.interfaces import ICriterion
from Products.CMFTopic.permissions import ChangeTopics
from Products.CMFTopic.permissions import View
from Products.CMFTopic.Topic import Topic


class SimpleStringCriterion( AbstractCriterion ):

    """ Represent a simple field-match for a string value.
    """

    implements(ICriterion)

    meta_type = 'String Criterion'

    security = ClassSecurityInfo()

    _editableAttributes = ( 'value', )

    def __init__(self, id, field):
        self.id = id
        self.field = field
        self.value = ''

    security.declareProtected( ChangeTopics, 'getEditForm' )
    def getEditForm( self ):
        """
            Return the skinned name of the edit form.
        """
        return 'ssc_edit'

    security.declareProtected( ChangeTopics, 'edit' )
    def edit( self, value ):
        """
            Update the value we are to match up against.
        """
        self.value = str( value )

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems( self ):
        """
            Return a sequence of criteria items, used by Topic.buildQuery.
        """
        result = []

        if self.value is not '':
            result.append( ( self.field, self.value ) )

        return tuple( result )

InitializeClass( SimpleStringCriterion )

# Register as a criteria type with the Topic class
Topic._criteriaTypes.append( SimpleStringCriterion )



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" CMFTopic product interfaces.

$Id: __init__.py 73215 2007-03-16 08:55:04Z yuppie $
"""

from zope.interface import Interface


class ITopic(Interface):

    """ Topics are 'canned queries'.

    o Each topic holds a set of zero or more Criteria objects specifying
      the query.
    """

    def queryCatalog(REQUEST=None, **kw):
        """ Invoke the catalog using our criteria.

        o Built-in criteria update any criteria passed in 'kw'.
        """


class IMutableTopic(ITopic):

    """ Updatable form of ITopic.
    """


class ICriterion(Interface):
    """\
    A Topic is composed of Criterion objects which specify the query
    used for the Topic.  By supplying some basic information, the
    Criterion objects can be plugged into Topics without the Topic
    having to be too aware of the Criteria types.
    """

    def Type():
        """\
        Return the type of criterion object this is (ie - 'List Criterion')
        """

    def Field():
        """\
        Return the field this criterion object searches on.
        """

    def Description():
        """\
        Return a brief description of the criteria type.
        """

    def editableAttributes():
        """\
        Returns a tuble of editable attributes.  The values of this
        are used by the topic to build commands to send to the
        'edit' method based on each criterion's setup.
        """

    def getEditForm():
        """\
        Return the name of a DTML component used to edit criterion.
        Editforms should be specific to their type of criteria.
        """

    def apply(command):
        """\
        To make it easier to apply values from the rather dynamic
        Criterion edit form using Python Scripts, apply takes a
        mapping object as a default and applies itself to self.edit.

        It's basically a nice and protected wrapper around
        self.edit(**command).
        """

# XXX: Interfaces have to specify the signature.
##    def edit(**kw):
##        """\
##        The signature of this method should be specific to the
##        criterion.  Using the values in the attribute
##        '_editableAttributes', the Topic can apply the right
##        commands to each criteria object as its being edited without
##        having to know too much about the structure.
##        """

    def getCriteriaItems():
        """\
        Return a sequence of key-value tuples, each representing
        a value to be injected into the query dictionary (and,
        therefore, tailored to work with the catalog).
        """



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Home of the abstract Criterion base class.

$Id: AbstractCriterion.py 94489 2009-01-03 16:28:19Z yuppie $
"""

from AccessControl.SecurityInfo import ClassSecurityInfo
from Acquisition import Implicit
from App.class_init import InitializeClass
from OFS.SimpleItem import Item
from Persistence import Persistent

from Products.CMFTopic.permissions import AccessContentsInformation
from Products.CMFTopic.permissions import ChangeTopics


class AbstractCriterion( Persistent, Item, Implicit ):
    """
        Abstract base class for Criterion objects.
    """

    security = ClassSecurityInfo()

    security.declareProtected(ChangeTopics, 'apply')
    def apply( self, command ):
        """
            Apply 'command', which is expected to be a dictionary,
            to 'self.edit' (makes using Python Scripts easier).
        """
        self.edit(**command)

    security.declareProtected( ChangeTopics, 'editableAttributes' )
    def editableAttributes( self ):
        """
            Return a list of editable attributes, used by topics
            to build commands to send to the 'edit' command of each
            criterion, which may vary.

            Requires concrete subclasses to implement the attribute
            '_editableAttributes' which is a tuple of attributes
            that can be edited, for example:

            _editableAttributes = ( 'value', 'direction' )
        """
        return self._editableAttributes

    security.declareProtected( AccessContentsInformation, 'Type' )
    def Type( self ):
        """
            Return the Type of Criterion this object is.  This
            method can be overriden in subclasses, or those
            concrete subclasses must define the 'meta_type'
            attribute.
        """
        return self.meta_type

    security.declareProtected( AccessContentsInformation, 'Field' )
    def Field( self ):
        """
            Return the field that this criterion searches on.  The
            concrete subclasses can override this method, or have
            the 'field' attribute.
        """
        return self.field

    security.declareProtected( AccessContentsInformation, 'Description' )
    def Description( self ):
        """
            Return a brief but helpful description of the Criterion type,
            preferably based on the classes __doc__ string.
        """
        lines = [ line.strip() for line in self.__doc__.splitlines() ]
        return ' '.join( [ line for line in lines if line ] )

InitializeClass( AbstractCriterion )



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for Topic module.

$Id: test_Topic.py 101695 2009-07-07 12:02:25Z yuppie $
"""

import unittest
from Testing import ZopeTestCase
ZopeTestCase.installProduct('CMFTopic', 1)

from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.User import UnrestrictedUser
from Acquisition import Implicit

from zope.component import getSiteManager
from zope.interface.verify import verifyClass
from zope.site.hooks import setSite

from Products.CMFCore.interfaces import ISyndicationTool
from Products.CMFCore.testing import ConformsToFolder
from Products.CMFCore.testing import EventZCMLLayer
from Products.CMFCore.tests.base.dummy import DummySite
from Products.CMFCore.tests.base.testcase import SecurityTest
from Products.CMFTopic.testing import FunctionalLayer


class FauxBrain( Implicit ):

    def __init__( self, object ):

        self._object = object

    def getObject( self ):

        return self._object


class DummyDocument( Implicit ):

    def __init__( self, id ):

        self._id = id

    def getId( self ):

        return self._id


class DummyCatalog( Implicit ):

    def __init__( self, index_ids=() ):

        self._objects = []
        self._index_ids = index_ids
        self._indexes = {}

        for index_id in index_ids:
            self._indexes[ index_id ] = {}

    def _index( self, obj, idxs=[] ):

        marker = object()
        self._objects.append( obj )

        rid = len( self._objects ) - 1

        for index_id in self._index_ids:

            value = getattr( obj, index_id, marker )

            if value is not marker:
                for word in value.split():
                    bucket = self._indexes[ index_id ].setdefault( word, [] )
                    bucket.append( rid )

    indexObject = _index

    reindexObject = _index

    def searchResults( self, REQUEST=None, **kw ):

        limit = None

        criteria = kw.copy()

        if REQUEST is not None:
            for k, v in REQUEST:
                criteria[ k ] = v

        results = set(range(len(self._objects)))

        for k, v in criteria.items():

            if k == 'sort_limit':
                limit = v

            else:
                results &= set(self._indexes[k].get(v, []))

        results = [ x for x in results ]

        if limit is not None:
            results = results[ :limit ]

        return [ FauxBrain( self._objects[ rid ] ) for rid in results ]


class DummySyndicationTool( Implicit ):

    def __init__( self, max_items ):

        self._max_items = max_items

    def getMaxItems( self, object ):

        return self._max_items


class TestTopic(ConformsToFolder, SecurityTest):

    """ Test all the general Topic cases.
    """

    layer = EventZCMLLayer

    def _getTargetClass(self):
        from Products.CMFTopic.Topic import Topic

        return Topic

    def _makeOne(self, id, *args, **kw):
        return self.site._setObject(id,
                                    self._getTargetClass()(id, *args, **kw))

    def _initSite(self, max_items=15, index_ids=()):
        sm = getSiteManager()
        self.site.portal_catalog = DummyCatalog( index_ids )
        self.site.portal_syndication = DummySyndicationTool( max_items )
        sm.registerUtility(self.site.portal_syndication, ISyndicationTool)

    def _initDocuments(self, **kw):
        for k, v in kw.items():

            document = DummyDocument( k )
            document.description = v

            self.site._setObject( k, v )
            self.site.portal_catalog._index( document )

    def setUp(self):
        SecurityTest.setUp(self)
        self.site = DummySite('site').__of__(self.root)

    def test_interfaces(self):
        from OFS.interfaces import IOrderedContainer
        from Products.CMFCore.interfaces import IContentish
        from Products.CMFTopic.interfaces import IMutableTopic
        from Products.CMFTopic.interfaces import ITopic

        verifyClass(IContentish, self._getTargetClass())
        verifyClass(IMutableTopic, self._getTargetClass())
        verifyClass(ITopic, self._getTargetClass())
        verifyClass(IOrderedContainer, self._getTargetClass())

    def test_Empty( self ):
        topic = self._makeOne('top')

        query = topic.buildQuery()
        self.assertEqual( len( query ), 0 )

    def test_Simple( self ):
        topic = self._makeOne('top')
        topic.addCriterion( 'foo', 'String Criterion' )
        topic.getCriterion( 'foo' ).edit( 'bar' )

        query = topic.buildQuery()
        self.assertEqual( len(query), 1 )
        self.assertEqual( query['foo'], 'bar' )

        topic.addCriterion( 'baz', 'Integer Criterion' )
        topic.getCriterion( 'baz' ).edit( 43 )

        query = topic.buildQuery()
        self.assertEqual( len( query ), 2 )
        self.assertEqual( query[ 'foo' ], 'bar' )
        self.assertEqual( query[ 'baz' ], 43 )

    def test_selfIndexing(self):
        # The Topic object is CatalogAware and should be in the catalog
        # after it has beeen instantiated.
        self._initSite()
        topic = self._makeOne('top')

        # A topic without criteria will return a full catalog search result
        # set, so we should not have one result, for the Topic object itself.
        results = topic.queryCatalog()

        self.assertEquals(len(results), 1)
        self.assertEquals(results[0].getObject().getId(), topic.getId())
        self.assertEquals(results[0].getObject(), topic)

    def test_searchableText(self):
        # Test the catalog helper
        topic = self._makeOne('top')
        topic.edit(False, title='FOO', description='BAR')

        st = topic.SearchableText()
        self.failUnless(st.find('BAR') != -1)
        self.failUnless(st.find('FOO') != -1)

    def test_queryCatalog_noop( self ):

        self._initSite()
        self._initDocuments( **_DOCUMENTS )
        topic = self._makeOne('top')

        # Need to filter out the Topic object itself, which is also
        # CatalogAware and will index itself after instantiation.
        brains = [ x for x in topic.queryCatalog()
                      if x.getObject().getId() != 'top' ]

        self.assertEqual( len( brains ), len( _DOCUMENTS ) )

        objects = [ brain.getObject() for brain in brains ]

        for object in objects:
            self.failUnless( object.getId() in _DOCUMENTS.keys() )
            self.failUnless( object.description in _DOCUMENTS.values() )

    def test_queryCatalog_simple( self ):

        WORD = 'something'

        self._initSite( index_ids=( 'description', ) )
        self._initDocuments( **_DOCUMENTS )
        topic = self._makeOne('top')

        topic.addCriterion( 'description', 'String Criterion' )
        topic.getCriterion( 'description' ).edit( WORD )

        brains = topic.queryCatalog()

        self.assertEqual( len( brains )
                        , len( [ x for x in _DOCUMENTS.values()
                                  if WORD in x ] ) )

        objects = [ brain.getObject() for brain in brains ]

        for object in objects:
            self.failUnless( object.getId() in _DOCUMENTS.keys() )
            self.failUnless( object.description in _DOCUMENTS.values() )

    def test_synContentValues_simple( self ):

        self._initSite()
        self._initDocuments( **_DOCUMENTS )
        topic = self._makeOne('top')

        #brains = topic.synContentValues()
        # Need to filter out the Topic object itself, which is also
        # CatalogAware and will index itself after instantiation.
        brains = [ x for x in topic.synContentValues()
                      if x.getObject().getId() != 'top' ]

        self.assertEqual( len( brains ), len( _DOCUMENTS ) )

        objects = [ brain.getObject() for brain in brains ]

        for object in objects:
            self.failUnless( object.getId() in _DOCUMENTS.keys() )
            self.failUnless( object.description in _DOCUMENTS.values() )

    def test_synContentValues_limit( self ):

        LIMIT = 3

        self._initSite( max_items=LIMIT )
        self._initDocuments( **_DOCUMENTS )
        topic = self._makeOne('top')

        brains = topic.synContentValues()

        self.assertEqual( len( brains ), LIMIT )

        objects = [ brain.getObject() for brain in brains ]

        for object in objects:
            self.failUnless( object.getId() in _DOCUMENTS.keys() )
            self.failUnless( object.description in _DOCUMENTS.values() )


class TestTopicFunctional(ZopeTestCase.FunctionalTestCase):

    layer = FunctionalLayer

    def afterSetUp(self):
        setSite(self.app.site)
        self.app.site.setupCurrentSkin(self.app.REQUEST)
        newSecurityManager(None, UnrestrictedUser('god', '', ['Manager'], ''))

    def test_Nested( self ):
        self.app.site.invokeFactory('Topic', 'top')
        topic = self.app.site.top

        topic.addCriterion( 'foo', 'String Criterion' )
        topic.getCriterion( 'foo' ).edit( 'bar' )

        topic.addSubtopic( 'qux' )
        subtopic = topic.qux

        subtopic.addCriterion( 'baz', 'String Criterion' )
        subtopic.getCriterion( 'baz' ).edit( 'bam' )

        query = subtopic.buildQuery()
        self.assertEqual( len( query ), 2 )
        self.assertEqual( query['foo'], 'bar' )
        self.assertEqual( query['baz'], 'bam' )

        subtopic.acquireCriteria = 0
        query = subtopic.buildQuery()
        self.assertEqual( len( query ), 1 )
        self.assertEqual( query['baz'], 'bam' )


_DOCUMENTS = \
{ 'one'     : "something in the way she moves"
, 'two'     : "I don't know much about history"
, 'three'   : "something about Mary"
, 'four'    : "something tells me I'm in love"
, 'five'    : "there's a certain wonderful something"
, 'six'     : "gonna wash that man right out of my hair"
, 'seven'   : "I'm so much in love"
}


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(TestTopic),
        unittest.makeSuite(TestTopicFunctional),
        ))

if __name__ == '__main__':
    from Products.CMFCore.testing import run
    run(test_suite())



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Allow topic to specify sorting.

$Id: SortCriterion.py 94489 2009-01-03 16:28:19Z yuppie $
"""

from AccessControl.SecurityInfo import ClassSecurityInfo
from App.class_init import InitializeClass
from zope.interface import implements

from Products.CMFTopic.AbstractCriterion import AbstractCriterion
from Products.CMFTopic.interfaces import ICriterion
from Products.CMFTopic.permissions import ChangeTopics
from Products.CMFTopic.permissions import View
from Products.CMFTopic.Topic import Topic


class SortCriterion( AbstractCriterion ):

    """
        Represent a mock criterion, to allow spelling the sort order
        and reversal items in a catalog query.
    """

    implements(ICriterion)

    meta_type = 'Sort Criterion'

    security = ClassSecurityInfo()

    field = None # Don't prevent use of field in other criteria

    _editableAttributes = ( 'reversed', )

    def __init__( self, id, index ):
        self.id = id
        self.index = index
        self.reversed = 0

    # inherit permissions
    def Field( self ):
        """
            Map the stock Criterion interface.
        """
        return self.index

    security.declareProtected( ChangeTopics, 'getEditForm' )
    def getEditForm( self ):
        """
            Return the name of skin method which renders the form
            used to edit this kind of criterion.
        """
        return 'sort_edit'

    security.declareProtected( ChangeTopics, 'edit' )
    def edit( self, reversed ):
        """
            Update the value we are to match up against.
        """
        self.reversed = bool(reversed)

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems( self ):
        """
            Return a tuple of query elements to be passed to the catalog
            (used by 'Topic.buildQuery()').
        """
        result = [ ( 'sort_on', self.index ) ]

        if self.reversed:
            result.append( ( 'sort_order', 'reverse' ) )

        return tuple( result )

InitializeClass( SortCriterion )

# Register as a criteria type with the Topic class
Topic._criteriaTypes.append( SortCriterion )



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for SimpleIntCriterion module.

$Id: test_SIC.py 91220 2008-09-17 21:15:09Z tseaver $
"""

import unittest
import Testing

from common import CriterionTestCase


class SimpleIntCriterionTests(CriterionTestCase, unittest.TestCase):

    def _getTargetClass(self):
        from Products.CMFTopic.SimpleIntCriterion import SimpleIntCriterion

        return SimpleIntCriterion

    def test_Empty( self ):
        sic = self._makeOne('foo', 'foofield')
        self.assertEqual( sic.getId(), 'foo' )
        self.assertEqual( sic.field, 'foofield' )
        self.assertEqual( sic.value, None )
        self.assertEqual( sic.getValueString(), '' )
        self.assertEqual( len(sic.getCriteriaItems() ), 0 )

    def test_EditWithString( self ):
        sic = self._makeOne('foo', 'foofield')
        sic.edit('0')
        self.assertEqual( sic.value, 0 )
        self.assertEqual( sic.getValueString(), '0' )

        items = sic.getCriteriaItems()
        self.assertEqual( len( items ), 1 )
        self.assertEqual( len( items[0] ), 2 )
        self.assertEqual( items[0][0], 'foofield' )
        self.assertEqual( items[0][1], 0 )

    def test_EditWithInt( self ):
        sic = self._makeOne('foo', 'foofield')
        sic.edit( 32 )
        self.assertEqual( sic.value, 32 )
        self.assertEqual( sic.getValueString(), '32' )

        items = sic.getCriteriaItems()
        self.assertEqual( len(items), 1 )
        self.assertEqual( len(items[0]), 2 )
        self.assertEqual( items[0][1], 32 )

    def test_RangeMin( self ):
        sic = self._makeOne('foo', 'foofield')
        sic.edit( '32', self._getTargetClass().MINIMUM )

        self.assertEqual( sic.value, 32 )
        self.assertEqual( sic.getValueString(), '32' )

        items = sic.getCriteriaItems()
        self.assertEqual( len(items), 1 )
        self.assertEqual( len( items[0] ), 2 )
        self.assertEqual( items[0][0], 'foofield' )
        self.assertEqual( items[0][1]['query'], 32 )
        self.assertEqual( items[0][1]['range'], 'min' )

    def test_RangeMin_withInt( self ):
        sic = self._makeOne('foo', 'foofield')
        sic.edit( 32, self._getTargetClass().MINIMUM )

        self.assertEqual( sic.value, 32 )
        self.assertEqual( sic.getValueString(), '32' )

        items = sic.getCriteriaItems()
        self.assertEqual( len(items), 1 )
        self.assertEqual( len( items[0] ), 2 )
        self.assertEqual( items[0][0], 'foofield' )
        self.assertEqual( items[0][1]['query'], 32 )
        self.assertEqual( items[0][1]['range'], 'min' )

    def test_RangeMax( self ):
        sic = self._makeOne('foo', 'foofield')
        sic.edit( '32', self._getTargetClass().MAXIMUM )

        self.assertEqual( sic.value, 32 )
        self.assertEqual( sic.getValueString(), '32' )

        items = sic.getCriteriaItems()
        self.assertEqual( len(items), 1 )
        self.assertEqual( len( items[0] ), 2 )
        self.assertEqual( items[0][0], 'foofield' )
        self.assertEqual( items[0][1]['query'], 32 )
        self.assertEqual( items[0][1]['range'], 'max' )

    def test_RangeMax_withInt( self ):
        sic = self._makeOne('foo', 'foofield')
        sic.edit( 32, self._getTargetClass().MAXIMUM )

        self.assertEqual( sic.value, 32 )
        self.assertEqual( sic.getValueString(), '32' )

        items = sic.getCriteriaItems()
        self.assertEqual( len(items), 1 )
        self.assertEqual( len( items[0] ), 2 )
        self.assertEqual( items[0][0], 'foofield' )
        self.assertEqual( items[0][1]['query'], 32 )
        self.assertEqual( items[0][1]['range'], 'max' )

    def test_RangeMinMax( self ):
        sic = self._makeOne('foo', 'foofield')
        sic.edit( '32 34', self._getTargetClass().MINMAX )

        self.assertEqual( sic.value, ( 32, 34 ) )
        self.assertEqual( sic.getValueString(), '32 34' )

        items = sic.getCriteriaItems()
        self.assertEqual( len(items), 1 )
        self.assertEqual( len( items[0] ), 2 )
        self.assertEqual( items[0][0], 'foofield' )
        self.assertEqual( items[0][1]['query'], ( 32, 34 ) )
        self.assertEqual( items[0][1]['range'], 'min:max' )

    def test_RangeMinMax_withTuple( self ):
        sic = self._makeOne('foo', 'foofield')
        sic.edit( ( 32, 34 ), self._getTargetClass().MINMAX )

        self.assertEqual( sic.value, ( 32, 34 ) )
        self.assertEqual( sic.getValueString(), '32 34' )

        items = sic.getCriteriaItems()
        self.assertEqual( len(items), 1 )
        self.assertEqual( len( items[0] ), 2 )
        self.assertEqual( items[0][0], 'foofield' )
        self.assertEqual( items[0][1]['query'], ( 32, 34 ) )
        self.assertEqual( items[0][1]['range'], 'min:max' )


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(SimpleIntCriterionTests),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Simple int-matching criterion

$Id: SimpleIntCriterion.py 94489 2009-01-03 16:28:19Z yuppie $
"""

from AccessControl.SecurityInfo import ClassSecurityInfo
from App.class_init import InitializeClass
from zope.interface import implements

from Products.CMFTopic.AbstractCriterion import AbstractCriterion
from Products.CMFTopic.interfaces import ICriterion
from Products.CMFTopic.permissions import ChangeTopics
from Products.CMFTopic.permissions import View
from Products.CMFTopic.Topic import Topic


class SimpleIntCriterion( AbstractCriterion ):

    """
        Represent a simple field-match for an integer value, including
        catalog range searches.
    """

    implements(ICriterion)

    meta_type = 'Integer Criterion'

    security = ClassSecurityInfo()
    _editableAttributes = ( 'value', 'direction' )

    MINIMUM = 'min'
    MAXIMUM = 'max'
    MINMAX = 'min:max'

    def __init__(self, id, field):
        self.id = id
        self.field = field
        self.value = self.direction = None

    security.declareProtected( ChangeTopics, 'getEditForm' )
    def getEditForm( self ):
        """
            Return the name of skin method which renders the form
            used to edit this kind of criterion.
        """
        return 'sic_edit'

    security.declareProtected( ChangeTopics, 'getValueString' )
    def getValueString( self ):
        """
            Return a string representation of the value for which this
            criterion filters.
        """
        if self.value is None:
            return ''

        if self.direction == self.MINMAX:

            value = self.value

            if type( value ) is not type( () ):
                value = ( value, value )

            return '%s %s' % value

        return str( self.value )

    security.declareProtected( ChangeTopics, 'edit' )
    def edit( self, value, direction=None ):
        """
            Update the value to be filtered, and the "direction" qualifier.
        """

        if type( value ) == type( '' ):
           value = value.strip()

        if not value:
            # An empty string was passed in, which evals to None
            self.value = self.direction = None

        elif direction:

            if direction == self.MINMAX:

                if type( value ) == type( '' ):
                    minimum, maximum = value.split(' ')
                else:
                    minimum, maximum = value

                self.value = ( int( minimum ), int( maximum ) )

            else:
                self.value = int( value )

            self.direction = direction

        else:
            self.value = int( value )
            self.direction = None

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems( self ):
        """
            Return a tuple of query elements to be passed to the catalog
            (used by 'Topic.buildQuery()').
        """
        if self.value is None:
            return ()
        elif self.direction is None:
            return ( ( self.Field(), self.value ), )
        else:
            return ( ( self.Field(), {'query': self.value,
                                      'range': self.direction} ), )

InitializeClass( SimpleIntCriterion )


# Register as a criteria type with the Topic class
Topic._criteriaTypes.append( SimpleIntCriterion )



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for SortCriterion module.

$Id: test_SortC.py 91220 2008-09-17 21:15:09Z tseaver $
"""

import unittest
import Testing

from common import CriterionTestCase


class SortCriterionTests(CriterionTestCase, unittest.TestCase):

    def _getTargetClass(self):
        from Products.CMFTopic.SortCriterion import SortCriterion

        return SortCriterion

    def test_Empty( self ):
        ssc = self._makeOne('foo', 'foofield')

        self.assertEqual( ssc.getId(), 'foo' )
        self.assertEqual( ssc.field, None )
        self.assertEqual( ssc.index, 'foofield' )
        self.assertEqual( ssc.Field(), 'foofield' )
        self.assertEqual( ssc.reversed, 0 )

        items = ssc.getCriteriaItems()
        self.assertEqual( len( items ), 1 )
        self.assertEqual( items[0][0], 'sort_on' )
        self.assertEqual( items[0][1], 'foofield' )

    def test_Nonempty( self ):
        ssc = self._makeOne('foo', 'foofield')

        ssc.edit( 1 )

        self.assertEqual( ssc.getId(), 'foo' )
        self.assertEqual( ssc.field, None )
        self.assertEqual( ssc.index, 'foofield' )
        self.assertEqual( ssc.Field(), 'foofield' )
        self.assertEqual( ssc.reversed, 1 )

        items = ssc.getCriteriaItems()
        self.assertEqual( len( items ), 2 )
        self.assertEqual( items[0][0], 'sort_on' )
        self.assertEqual( items[0][1], 'foofield' )
        self.assertEqual( items[1][0], 'sort_order' )
        self.assertEqual( items[1][1], 'reverse' )


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(SortCriterionTests),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##parameters=b_start=0
##
from ZTUtils import Batch
from Products.CMFDefault.utils import decode


options = {}

subtopics = [ {'title': item.Title() or item.getId(),
               'url': item.absolute_url()}
              for item in context.contentValues() ]
options['listSubtopicInfos'] = subtopics

queries = context.buildQuery()
options['listQueries'] = tuple([ '%s: %s' % q for q in queries.items() ])

target = context.REQUEST['ACTUAL_URL']
items = context.queryCatalog()
batch_obj = Batch(items, 20, b_start, orphan=1)

items = [ {'creators': item.listCreators,
           'date': item.Date,
           'description': item.Description,
           'id': item.getId,
           'title': item.Title and ('(%s)' % item.Title) or '',
           'url': '%s/view' % item.getURL()}
          for item in batch_obj ]

navigation = context.getBatchNavigation(batch_obj, target)
options['batch'] = {'listItemInfos': items,
                    'navigation': navigation}

return context.topic_view_template(**decode(options, script))



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for DateCriteria module.

$Id: test_DateC.py 101695 2009-07-07 12:02:25Z yuppie $
"""

import unittest
from Testing import ZopeTestCase

from DateTime.DateTime import DateTime
from zope.site.hooks import setSite

from Products.CMFCore.tests.base.dummy import DummyContent
from Products.CMFDefault.testing import FunctionalLayer
from Products.CMFTopic.Topic import Topic

from common import CriterionTestCase

def _replace_DC__as_of(new_callable):
    from Products.CMFTopic import DateCriteria
    old_value = DateCriteria._as_of
    DateCriteria._as_of = new_callable
    return old_value


class FriendlyDateCriterionTests(CriterionTestCase, unittest.TestCase):

    lessThanFiveDaysOld = { 'value': 5
                          , 'operation': 'max'
                          , 'daterange': 'old'
                          }

    lessThanOneMonthAhead = { 'value': 31
                            , 'operation': 'max'
                            , 'daterange': 'ahead'
                            }
    today = { 'value': 0
            , 'operation': 'within_day'
            , 'daterange': 'ahead'
            }

    def setUp(self):
        self._now = DateTime()
        self._old_as_of = _replace_DC__as_of(lambda: self._now)

    def tearDown(self):
        _replace_DC__as_of(self._old_as_of)

    def _getTargetClass(self):
        from Products.CMFTopic.DateCriteria import FriendlyDateCriterion

        return FriendlyDateCriterion

    def test_Empty( self ):
        friendly = self._makeOne('foo', 'foofield')

        self.assertEqual( friendly.getId(), 'foo' )
        self.assertEqual( friendly.field, 'foofield' )
        self.assertEqual( friendly.value, None )
        self.assertEqual( friendly.operation, 'min' )
        self.assertEqual( friendly.daterange, 'old' )
        self.assertEqual( len( friendly.getCriteriaItems() ), 0 )

    def test_ListOfDefaultDates( self ):
        friendly = self._makeOne('foo', 'foofield')

        d = friendly.defaultDateOptions()
        self.assertEqual( d[0][0], 0 )
        self.assertEqual( d[1][0], 1 )
        self.assertEqual( d[2][0], 2 )

    def test_Clear( self ):
        friendly = self._makeOne('foo', 'foofield')

        friendly.edit( value=None )
        self.assertEqual( friendly.value, None )
        self.assertEqual( friendly.operation, 'min' )
        self.assertEqual( friendly.daterange, 'old' )

    def test_Basic( self ):
        friendly = self._makeOne('foo', 'foofield')

        friendly.apply( self.lessThanFiveDaysOld )
        self.assertEqual( friendly.value, 5 )
        self.assertEqual( friendly.operation, 'max' )
        self.assertEqual( friendly.daterange, 'old' )

    def test_BadInput( self ):
        friendly = self._makeOne('foo', 'foofield')

        # Bogus value
        self.assertRaises( ValueError, friendly.edit, 'blah' )

        # Bogus operation
        self.assertRaises( ValueError, friendly.edit, 4, 'min:max', 'old' )

        # Bogus daterange
        self.assertRaises( ValueError, friendly.edit, 4, 'max', 'new' )

    def test_StringAsValue( self ):
        friendly = self._makeOne('foo', 'foofield')

        friendly.edit( '4' )
        self.assertEqual( friendly.value, 4 )

        friendly.edit( '-4' )
        self.assertEqual( friendly.value, -4 )

        friendly.edit( '' )
        self.assertEqual( friendly.value, None )

    def test_Today( self ):
        friendly = self._makeOne('foo', 'foofield')

        friendly.apply( self.today )
        self.assertEqual( friendly.daterange, 'ahead' )

        now = DateTime()

        result = friendly.getCriteriaItems()
        self.assertEqual( len(result), 1 )
        self.assertEqual( result[0][0], 'foofield' )
        self.assertEqual( result[0][1]['query'],
                          ( now.earliestTime(), now.latestTime() ) )
        self.assertEqual( result[0][1]['range'], 'min:max' )

    def test_FiveDaysOld( self ):
        # This should create a query
        friendly = self._makeOne('foo', 'foofield')

        friendly.apply( self.lessThanFiveDaysOld )
        self.assertEqual( friendly.daterange, 'old' )

        result = friendly.getCriteriaItems()
        self.assertEqual( len(result), 1 )
        self.assertEqual( result[0][0], 'foofield' )
        expect_earliest, expect_now = result[0][1]['query']
        self.assertEqual( expect_earliest.Date(),
                          ( DateTime() - 5 ).Date() )
        self.assertEqual( result[0][1]['range'], 'min:max' )

    def test_OneMonthAhead( self ):
        friendly = self._makeOne('foo', 'foofield')

        friendly.apply( self.lessThanOneMonthAhead )
        self.assertEqual( friendly.daterange, 'ahead' )

        result = friendly.getCriteriaItems()
        expect_now, expect_latest = result[0][1]['query']
        self.assertEqual( expect_latest.Date(), ( DateTime() + 31 ).Date() )
        self.assertEqual( expect_now.Date(), DateTime().Date() )
        self.assertEqual( result[0][1]['range'], 'min:max' )


class FriendlyDateCriterionFunctionalTests(ZopeTestCase.FunctionalTestCase):

    layer = FunctionalLayer

    # Test the date criterion using a "real CMF" with catalog etc.
    selectable_diffs = [0, 1, 2, 5, 7, 14, 31, 93, 186, 365, 730]
    nonzero_diffs = [1, 2, 5, 7, 14, 31, 93, 186, 365, 730]
    day_diffs = [-730, -365, -186, -93, -31, -14, -7, -5, -2, -1]
    day_diffs.extend(selectable_diffs)

    def afterSetUp(self):
        setSite(self.app.site)
        self.site = self.app.site
        self.site._setObject( 'topic', Topic('topic') )
        self.topic = self.site.topic
        self.topic.addCriterion('modified', 'Friendly Date Criterion')
        self.topic.addCriterion('portal_type', 'String Criterion')
        type_crit = self.topic.getCriterion('portal_type')
        type_crit.edit(value='Dummy Content')
        self.criterion = self.topic.getCriterion('modified')
        self.now = DateTime()
        self._old_as_of = _replace_DC__as_of(lambda: self.now)

        for i in self.day_diffs:
            dummy_id = 'dummy%i' % i
            self.site._setObject( dummy_id, DummyContent( id=dummy_id
                                                        , catalog=1
                                                        ) )
            dummy_ob = getattr(self.site, dummy_id)
            dummy_ob.modified_date = self.now + i
            dummy_ob.reindexObject()

    def beforeTearDown(self):
        _replace_DC__as_of(self._old_as_of)

    def test_Harness(self):
        # Make sure the test harness is set up OK
        ob_values = self.site.objectValues(['Dummy'])
        self.assertEqual(len(ob_values), len(self.day_diffs))

        catalog_results = self.site.portal_catalog(portal_type='Dummy Content')
        self.assertEqual(len(catalog_results), len(self.day_diffs))

    def test_WithinDayAgo(self):
        # What items were modified "On the day X days ago"
        for diff in self.selectable_diffs:
            self.criterion.edit( value=abs(diff)
                               , operation='within_day'
                               , daterange='old'
                               )
            results = self.topic.queryCatalog()

            # There is only one item with an modified date for this day
            self.assertEquals(len(results), 1)
            self.assertEquals( results[0].modified.Date()
                             , (self.now-diff).Date()
                             )

    def test_WithinDayAhead(self):
        # What items were modified "On the day X days ahead"
        for diff in self.selectable_diffs:
            self.criterion.edit( value=abs(diff)
                               , operation='within_day'
                               , daterange='ahead'
                               )
            results = self.topic.queryCatalog()

            # There is only one item with an modified date for this day
            self.assertEquals(len(results), 1)
            self.assertEquals( results[0].modified.Date()
                             , (self.now+diff).Date()
                             )

    def test_MoreThanDaysAgo(self):
        # What items are modified "More than X days ago"
        resultset_size = len(self.nonzero_diffs)

        for diff in self.nonzero_diffs:
            self.criterion.edit( value=diff
                               , operation='min'
                               , daterange='old'
                               )
            results = self.topic.queryCatalog()

            # As we move up in our date difference range, we must find as
            # many items as we have "modified" values <= the current value
            # in our sequence of user-selectable time differences. As we
            # increase the "value", we actually move backwards in time, so
            # the expected count of results *decreases*
            self.assertEquals(len(results), resultset_size)
            for brain in results:
                self.failUnless(brain.modified <= self.now-diff)

            resultset_size -= 1

    def test_MoreThanZeroDaysAgo(self):
        # What items are modified "More than 0 days ago"?
        # This represents a special case. The "special munging"
        # that corrects the query terms to what a human would expect
        # are not applied and the search is a simple
        # "everything in the future" search.
        resultset_size = len(self.selectable_diffs)
        self.criterion.edit( value=0
                           , operation='min'
                           , daterange='old'
                           )
        results = self.topic.queryCatalog()
        self.assertEquals(len(results), resultset_size)
        for brain in results:
            self.failUnless(brain.modified >= self.now)

    def test_MoreThanDaysAhead(self):
        # What items are modified "More than X days ahead"
        resultset_size = len(self.nonzero_diffs)

        for diff in self.nonzero_diffs:
            self.criterion.edit( value=diff
                               , operation='min'
                               , daterange='ahead'
                               )
            results = self.topic.queryCatalog()

            # As we move up in our date difference range, we must find as
            # many items as we have "modified" values >= the current value
            # in our sequence of user-selectable time differences. As we
            # increase the "value", we actually move formward in time, so
            # the expected count of results *decreases*
            self.assertEquals(len(results), resultset_size)
            for brain in results:
                self.failUnless(brain.modified >= self.now+diff)

            resultset_size -= 1

    def test_MoreThanZeroDaysAhead(self):
        # What items are modified "More than 0 days ahead"?
        # This represents a special case. The "special munging"
        # that corrects the query terms to what a human would expect
        # are not applied and the search is a simple
        # "everything in the future" search.
        resultset_size = len(self.selectable_diffs)
        self.criterion.edit( value=0
                           , operation='min'
                           , daterange='ahead'
                           )
        results = self.topic.queryCatalog()
        self.assertEquals(len(results), resultset_size)
        for brain in results:
            self.failUnless(brain.modified >= self.now)

    def test_LessThanDaysAgo(self):
        # What items are modified "Less than X days ago"
        resultset_size = 2

        for diff in self.nonzero_diffs:
            self.criterion.edit( value=diff
                               , operation='max'
                               , daterange='old'
                               )
            results = self.topic.queryCatalog()

            # With this query we are looking for items modified "less than
            # X days ago", meaning between the given time and now. As we move
            # through the selectable day values we increase the range to
            # search through and thus increase the resultset size.
            self.assertEquals(len(results), resultset_size)
            for brain in results:
                self.failUnless(self.now-diff <= brain.modified <= self.now)

            resultset_size += 1

    def test_LessThanZeroDaysAgo(self):
        # What items are modified "Less than 0 days ago"?
        # This represents a special case. The "special munging"
        # that corrects the query terms to what a human would expect
        # are not applied and the search is a simple
        # "everything in the past" search.
        resultset_size = len(self.selectable_diffs)
        self.criterion.edit( value=0
                           , operation='max'
                           , daterange='old'
                           )
        results = self.topic.queryCatalog()
        self.assertEquals(len(results), resultset_size)
        for brain in results:
            self.failUnless(brain.modified <= self.now)

    def test_LessThanDaysAhead(self):
        # What items are modified "Less than X days ahead"
        resultset_size = 2

        for diff in self.nonzero_diffs:
            self.criterion.edit( value=diff
                               , operation='max'
                               , daterange='ahead'
                               )
            results = self.topic.queryCatalog()

            # With this query we are looking for items modified "less than
            # X days ahead", meaning between now and the given time. As we move
            # through the selectable day values we increase the range to
            # search through and thus increase the resultset size.
            self.assertEquals(len(results), resultset_size)
            for brain in results:
                self.failUnless(self.now+diff >= brain.modified >= self.now)

            resultset_size += 1

    def test_LessThanZeroDaysAhead(self):
        # What items are modified "Less than 0 days ahead"?
        # This represents a special case. The "special munging"
        # that corrects the query terms to what a human would expect
        # are not applied and the search is a simple
        # "everything in the past" search.
        resultset_size = len(self.selectable_diffs)
        self.criterion.edit( value=0
                           , operation='max'
                           , daterange='ahead'
                           )
        results = self.topic.queryCatalog()
        self.assertEquals(len(results), resultset_size)
        for brain in results:
            self.failUnless(brain.modified <= self.now)


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(FriendlyDateCriterionTests),
        unittest.makeSuite(FriendlyDateCriterionFunctionalTests),
        ))

if __name__ == '__main__':
    from Products.CMFCore.testing import run
    run(test_suite())



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Topic: Canned catalog queries

$Id: Topic.py 94489 2009-01-03 16:28:19Z yuppie $
"""

from AccessControl.SecurityInfo import ClassSecurityInfo
from Acquisition import aq_parent, aq_inner
from App.class_init import InitializeClass
from zope.component import getUtility
from zope.component.factory import Factory
from zope.interface import implements

from Products.CMFCore.interfaces import ISyndicationTool
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.SkinnedFolder import SkinnedFolder
from Products.CMFTopic.interfaces import IMutableTopic
from Products.CMFTopic.interfaces import ITopic
from Products.CMFTopic.permissions import View
from Products.CMFTopic.permissions import AddTopics
from Products.CMFTopic.permissions import ChangeTopics

def addTopic(self, id, title='', REQUEST=None):
    """ Create an empty topic.
    """
    topic = Topic( id )
    topic.id = id
    topic.title = title
    self._setObject(id, topic, suppress_events=True)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect( 'manage_main' )


class Topic(SkinnedFolder):

    """ Topics are 'canned queries'.

    o Each topic holds a set of zero or more Criteria objects specifying
      the query.
    """

    implements(IMutableTopic, ITopic)

    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    acquireCriteria = True
    _criteriaTypes = []

    security.declareProtected(ChangeTopics, 'listCriteria')
    def listCriteria(self):
        """ Return a list of our criteria objects.
        """
        return self.objectValues( self._criteria_metatype_ids() )

    security.declareProtected(ChangeTopics, 'listCriteriaTypes')
    def listCriteriaTypes(self):
        """ List the available criteria types.
        """
        out = []

        for ct in self._criteriaTypes:
            out.append( { 'name': ct.meta_type } )

        return out

    security.declareProtected(ChangeTopics, 'listAvailableFields')
    def listAvailableFields(self):
        """ Return a list of available fields for new criteria.
        """
        portal_catalog = getToolByName( self, 'portal_catalog' )
        currentfields = map( lambda x: x.Field(), self.listCriteria() )
        availfields = filter(
            lambda field, cf=currentfields: field not in cf,
            portal_catalog.indexes()
            )
        return availfields

    security.declareProtected(ChangeTopics, 'listSubtopics')
    def listSubtopics(self):
        """ Return a list of our subtopics.
        """
        return self.objectValues( self.meta_type )

    security.declareProtected(ChangeTopics, 'edit')
    def edit(self, acquireCriteria, title=None, description=None):
        """ Set the flag which indicates whether to acquire criteria.

        o If set, reuse creiteria from parent topics;

        o Also update metadata about the Topic.
        """
        self.acquireCriteria = bool(acquireCriteria)
        if title is not None:
            self.title = title
        self.description = description

        self.reindexObject()

    security.declareProtected(View, 'buildQuery')
    def buildQuery(self):
        """ Construct a catalog query using our criterion objects.
        """
        result = {}

        if self.acquireCriteria:

            try:
                # Tracker 290 asks to allow combinations, like this:
                # parent = aq_parent( self )
                parent = aq_parent( aq_inner( self ) )
                result.update( parent.buildQuery() )

            except: # oh well, can't find parent, or it isn't a Topic.
                pass

        for criterion in self.listCriteria():

            for key, value in criterion.getCriteriaItems():
                result[ key ] = value

        return result

    security.declareProtected(View, 'queryCatalog')
    def queryCatalog(self, REQUEST=None, **kw):
        """ Invoke the catalog using our criteria.

        o Built-in criteria update any criteria passed in 'kw'.
        """
        kw.update( self.buildQuery() )
        portal_catalog = getToolByName( self, 'portal_catalog' )
        return portal_catalog.searchResults(REQUEST, **kw)

    security.declareProtected(View, 'synContentValues')
    def synContentValues(self):
        """ Return a limited subset of the brains for our query.

        o Return no more brain objects than the limit set by the
          syndication tool.
        """
        syn_tool = getUtility(ISyndicationTool)
        limit = syn_tool.getMaxItems( self )
        brains = self.queryCatalog( sort_limit=limit )[ :limit ]
        return [ brain.getObject() for brain in brains ]

    ### Criteria adding/editing/deleting
    security.declareProtected(ChangeTopics, 'addCriterion')
    def addCriterion(self, field, criterion_type):
        """ Add a new search criterion.
        """
        crit = None
        newid = 'crit__%s' % field

        for ct in self._criteriaTypes:

            if criterion_type == ct.meta_type:
                crit = ct( newid, field )

        if crit is None:
            # No criteria type matched passed in value
            raise NameError, 'Unknown Criterion Type: %s' % criterion_type

        self._setObject( newid, crit )

    security.declareProtected(ChangeTopics, 'deleteCriterion')
    def deleteCriterion(self, criterion_id):
        """ Delete selected criterion.
        """
        if type( criterion_id ) is type( '' ):
            self._delObject( criterion_id )
        elif type( criterion_id ) in ( type( () ), type( [] ) ):
            for cid in criterion_id:
                self._delObject( cid )

    security.declareProtected(View, 'getCriterion')
    def getCriterion(self, criterion_id):
        """ Get the criterion object.
        """
        try:
            return self._getOb( 'crit__%s' % criterion_id )
        except AttributeError:
            return self._getOb( criterion_id )

    security.declareProtected(AddTopics, 'addSubtopic')
    def addSubtopic(self, id):
        """ Add a new subtopic.
        """
        ti = self.getTypeInfo()
        ti.constructInstance(self, id)
        return self._getOb( id )

    #
    #   Helper methods
    #
    security.declarePrivate('_criteria_metatype_ids')
    def _criteria_metatype_ids(self):

        result = []

        for mt in self._criteriaTypes:
            result.append( mt.meta_type )

        return tuple( result )

InitializeClass(Topic)

TopicFactory = Factory(Topic)



##############################################################################
#
# Copyright (c) 2005 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" GenericSetup export / import support for topics / criteria.

$Id: exportimport.py 94391 2008-12-27 21:56:17Z tseaver $
"""

from xml.dom.minidom import parseString

from zope.interface import implements

from Products.GenericSetup.interfaces import IFilesystemExporter
from Products.GenericSetup.interfaces import IFilesystemImporter
from Products.GenericSetup.content import FolderishExporterImporter
try:
    from Products.GenericSetup.utils import PageTemplateResource
except ImportError: # BBB:  no egg support
    from Products.PageTemplates.PageTemplateFile \
        import PageTemplateFile as PageTemplateResource

from Products.CMFTopic.Topic import Topic

class TopicExportImport(FolderishExporterImporter):
    """ Dump topic criteria to / from an XML file.
    """
    implements(IFilesystemExporter, IFilesystemImporter)

    encoding = None
    _FILENAME = 'criteria.xml'
    _ROOT_TAGNAME = 'criteria'

    def __init__(self, context):
        self.context = context

    def listExportableItems(self):
        """ See IFilesystemExporter.
        """
        criteria_metatypes = self.context._criteria_metatype_ids()
        return [x for x in FolderishExporterImporter.listExportableItems(self)
                   if x[1].meta_type not in criteria_metatypes]

    def export(self, export_context, subdir, root=False):
        """ See IFilesystemExporter.
        """
        FolderishExporterImporter.export(self, export_context, subdir, root)
        template = PageTemplateResource('xml/%s' % self._FILENAME,
                                        globals()).__of__(self.context)
        export_context.writeDataFile('%s/criteria.xml' % self.context.getId(),
                                     template(info=self._getExportInfo()),
                                     'text/xml',
                                     subdir,
                                    )

    def import_(self, import_context, subdir, root=False):
        """ See IFilesystemImporter
        """
        FolderishExporterImporter.import_(self, import_context, subdir, root)

        self.encoding = import_context.getEncoding()

        if import_context.shouldPurge():
            self._purgeContext()

        data = import_context.readDataFile('%s/criteria.xml'
                                                % self.context.getId(),
                                           subdir)

        if data is not None:
            dom = parseString(data)
            root = dom.firstChild
            assert root.tagName == self._ROOT_TAGNAME
            self._updateFromDOM(root)

    def _getNodeAttr(self, node, attrname, default=None):
        attr = node.attributes.get(attrname)
        if attr is None:
            return default
        value = attr.value
        if isinstance(value, unicode) and self.encoding is not None:
            value = value.encode(self.encoding)
        return value

    def _purgeContext(self):
        context = self.context
        criterion_ids = context.objectIds(context._criteria_metatype_ids())
        for criterion_id in criterion_ids:
            self.context._delObject(criterion_id)

    def _updateFromDOM(self, root):
        for criterion in root.getElementsByTagName('criterion'):
            c_type = self._getNodeAttr(criterion, 'type', None)
            field = self._getNodeAttr(criterion, 'field', None)
            attributes = {}
            for attribute in criterion.getElementsByTagName('attribute'):
                name = self._getNodeAttr(attribute, 'name', None)
                value = self._getNodeAttr(attribute, 'value', None)
                if name == 'reversed':
                    value = value in ('True', 'true', '1')
                attributes[name] = value

            self.context.addCriterion(field, c_type)
            added = self.context.getCriterion(field)
            added.edit(**attributes)

    def _getExportInfo(self):
        context = self.context
        criterion_info = []

        for criterion_id, criterion in context.objectItems(
                                        context._criteria_metatype_ids()):

            # SortCriterion stashes the 'field' as 'index'.
            field = getattr(criterion, 'index', criterion.field)

            info = {'criterion_id': criterion_id,
                    'type': criterion.meta_type,
                    'field': field,
                    'attributes': []
                   }

            attributes = info['attributes']
            for attrname in criterion.editableAttributes():
                value = getattr(criterion, attrname)
                if type(value) in (tuple, list):
                    value = ','.join(value)
                attributes.append((attrname, value))

            criterion_info.append(info)

        return {'criteria': criterion_info,
               }

    def _mustPreserve(self):
        context = self.context
        keepers = FolderishExporterImporter._mustPreserve(self)
        keepers.extend(context.objectItems(context._criteria_metatype_ids()))
        return keepers


class SubtopicFactory(object):

    def __init__(self, context):
        self.context = context

    def __call__(self, id):
        topic = Topic(id)
        topic.portal_type = 'Topic'
        return topic



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for ListCriterion module.

$Id: test_ListC.py 91220 2008-09-17 21:15:09Z tseaver $
"""

import unittest
import Testing

from common import CriterionTestCase


class ListCriterionTests(CriterionTestCase, unittest.TestCase):

    def _getTargetClass(self):
        from Products.CMFTopic.ListCriterion import ListCriterion

        return ListCriterion

    def test_Empty( self ):
        listc = self._makeOne('foo', 'foofield')

        self.assertEqual( listc.getId(), 'foo' )
        self.assertEqual( listc.field, 'foofield' )
        self.assertEqual( listc.value, ('',) )
        self.assertEqual( len(listc.getCriteriaItems()), 0 )

    def test_Edit_withString( self ):
        listc = self._makeOne('foo', 'foofield')

        listc.edit('bar\nbaz')
        self.assertEqual( listc.getId(), 'foo' )
        self.assertEqual( listc.field, 'foofield' )
        self.assertEqual( listc.value, ( 'bar', 'baz' ) )

        items = listc.getCriteriaItems()
        self.assertEqual( len( items ), 1 )
        self.assertEqual( len( items[0] ), 2 )
        self.assertEqual( items[0][0], 'foofield' )
        self.assertEqual( items[0][1], ( 'bar', 'baz' ) )

    def test_Edit_withList( self ):
        listc = self._makeOne('foo', 'foofield')

        abc = [ 'a', 'b', 'c' ]
        listc.edit( abc )

        items = listc.getCriteriaItems()
        self.failUnless( 'foofield' in map( lambda x: x[0], items ) )
        self.failUnless( tuple( abc ) in map( lambda x: x[1], items ) )

    def test_operator( self ):
        listc = self._makeOne('foo', 'foofield')

        abc = [ 'a', 'b', 'c' ]

        listc.edit( abc )
        items = listc.getCriteriaItems()
        self.assertEqual( len( items ), 1 )

        listc.edit( abc, 'or' )
        items = listc.getCriteriaItems()
        self.assertEqual( len( items ), 2 )
        self.failUnless( ( 'foofield_operator', 'or' ) in items )

        listc.edit( abc, 'and' )
        items = listc.getCriteriaItems()
        self.assertEqual( len( items ), 2 )
        self.failUnless( ( 'foofield_operator', 'and' ) in items )


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(ListCriterionTests),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Various date criteria

$Id: DateCriteria.py 94489 2009-01-03 16:28:19Z yuppie $
"""

from AccessControl.SecurityInfo import ClassSecurityInfo
from App.class_init import InitializeClass
from DateTime.DateTime import DateTime
from zope.interface import implements

from Products.CMFTopic.AbstractCriterion import AbstractCriterion
from Products.CMFTopic.interfaces import ICriterion
from Products.CMFTopic.permissions import ChangeTopics
from Products.CMFTopic.permissions import View
from Products.CMFTopic.Topic import Topic

_as_of = DateTime  # Allow for controlled value when testing


class FriendlyDateCriterion( AbstractCriterion ):

    """
        Put a friendly interface on date range searches, like
        'where effective date is less than 5 days old'.
    """

    implements(ICriterion)

    meta_type = 'Friendly Date Criterion'

    security = ClassSecurityInfo()

    _editableAttributes = ( 'value', 'operation', 'daterange' )

    _defaultDateOptions = ( (     0, 'Now'      )
                          , (     1, '1 Day'    )
                          , (     2, '2 Days'   )
                          , (     5, '5 Days'   )
                          , (     7, '1 Week'   )
                          , (    14, '2 Weeks'  )
                          , (    31, '1 Month'  )
                          , (  31*3, '3 Months' )
                          , (  31*6, '6 Months' )
                          , (   365, '1 Year'   )
                          , ( 365*2, '2 years'  )
                          )

    def __init__( self, id, field ):

        self.id = id
        self.field = field
        self.value = None
        self.operation = 'min'
        self.daterange = 'old'

    security.declarePublic( 'defaultDateOptions' )
    def defaultDateOptions( self ):
        """
            Return a list of default values and labels for date options.
        """
        return self._defaultDateOptions

    security.declareProtected( ChangeTopics, 'getEditForm' )
    def getEditForm( self ):
        """
            Return the name of the skin method used by Topic to edit
            criteria of this type.
        """
        return 'friendlydatec_editform'

    security.declareProtected( ChangeTopics, 'edit' )
    def edit( self
            , value=None
            , operation='min'
            , daterange='old'
            ):
        """
            Update the values to match against.
        """
        if value in ( None, '' ):
            self.value = None
        else:
            try:
                self.value = int( value )
            except:
                raise ValueError, 'Supplied value should be an int'

        if operation in ( 'min', 'max', 'within_day' ):
            self.operation = operation
        else:
            raise ValueError, 'Operation type not in set {min,max,within_day}'

        if daterange in ( 'old', 'ahead' ):
            self.daterange = daterange
        else:
            raise ValueError, 'Date range not in set {old,ahead}'

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems( self ):
        """
            Return a sequence of items to be used to build the catalog query.
        """
        if self.value is not None:
            field = self.Field()
            value = self.value
            operation = self.operation

            # Negate the value for 'old' days
            if self.daterange == 'old' and value != 0:
                value = -value

                # Also reverse the operator to match what a user would expect.
                # Queries such as "More than 2 days ago" should match dates
                # *earlier* than "today minus 2", and "Less than 2 days ago"
                # would be expected to return dates *later* then "today minus
                # two".
                if operation == 'max':
                    operation = 'min'
                elif operation == 'min':
                    operation = 'max'

            now = _as_of()
            date = now + value

            if operation == 'within_day':
                # When items within a day are requested, the range is between
                # the earliest and latest time of that particular day
                range = ( date.earliestTime(), date.latestTime() )
                return ( ( field, {'query': range, 'range': 'min:max'} ), )

            elif operation == 'min':
                if value != 0:
                    if self.daterange == 'old':
                        date_range = (date, now)
                        return ( ( field, { 'query': date_range
                                          , 'range': 'min:max'
                                          } ), )
                    else:
                        return ( ( field, { 'query': date.earliestTime()
                                          , 'range': operation
                                          } ), )
                else:
                    # Value 0 means "Now", so get everything from now on
                    return ( ( field, {'query': date,'range': operation } ), )

            elif operation == 'max':
                if value != 0:
                    if self.daterange == 'old':
                        return ((field, {'query': date, 'range': operation}),)
                    else:
                        date_range = (now, date.latestTime())
                        return ( ( field, { 'query': date_range
                                          , 'range': 'min:max'
                                          } ), )
                else:
                    # Value is 0, meaning "Now", get everything before "Now"
                    return ( ( field, {'query': date, 'range': operation} ), )
        else:
            return ()

InitializeClass(FriendlyDateCriterion)


# Register as a criteria type with the Topic class
Topic._criteriaTypes.append( FriendlyDateCriterion )



##parameters=title, description, acquireCriteria=False, **kw
##
from Products.CMFDefault.utils import Message as _

if title!=context.title or description != context.description or \
        acquireCriteria != context.acquireCriteria:
    context.edit(title=title, description=description,
                 acquireCriteria=acquireCriteria)
    return context.setStatus(True, _(u'Topic changed.'))
else:
    return context.setStatus(False, _(u'Nothing to change.'))



##############################################################################
#
# Copyright (c) 2005 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" CMFTopic product:  unit test utilities.

$Id: common.py 91220 2008-09-17 21:15:09Z tseaver $
"""

from unittest import TestCase

from zope.interface.verify import verifyClass


class CriterionTestCase:

    def _makeOne(self, id, *args, **kw):
        return self._getTargetClass()(id, *args, **kw)

    def test_interfaces(self):
        from Products.CMFTopic.interfaces import ICriterion

        verifyClass(ICriterion, self._getTargetClass())



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Topic: Canned catalog queries

$Id: __init__.py 94391 2008-12-27 21:56:17Z tseaver $
"""

def initialize(context):

    from Products.CMFCore.utils import ContentInit

    import Topic
    from permissions import AddTopics


    # Make sure security is initialized
    import DateCriteria
    import ListCriterion
    import SimpleIntCriterion
    import SimpleStringCriterion
    import SortCriterion

    context.registerHelpTitle( 'CMF Topic Help' )
    context.registerHelp( directory='help' )

    # BBB: register oldstyle constructors
    ContentInit( 'CMF Topic Content'
               , content_types=()
               , permission=AddTopics
               , extra_constructors=(Topic.addTopic,)
               , visibility=None
               ).initialize( context )



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for SimpleStringCriterion module.

$Id: test_SSC.py 91220 2008-09-17 21:15:09Z tseaver $
"""

import unittest
import Testing

from common import CriterionTestCase


class SimpleStringCriterionTests(CriterionTestCase, unittest.TestCase):

    def _getTargetClass(self):
        from Products.CMFTopic.SimpleStringCriterion \
                import SimpleStringCriterion

        return SimpleStringCriterion

    def test_Empty( self ):
        ssc = self._makeOne('foo', 'foofield')

        self.assertEqual( ssc.getId(), 'foo' )
        self.assertEqual( ssc.field, 'foofield' )
        self.assertEqual( ssc.value, '' )
        self.assertEqual( len( ssc.getCriteriaItems() ), 0 )

    def test_Nonempty( self ):
        ssc = self._makeOne('foo', 'foofield')
        ssc.edit( 'bar' )

        self.assertEqual( ssc.getId(), 'foo' )
        self.assertEqual( ssc.field, 'foofield' )
        self.assertEqual( ssc.value, 'bar' )

        items = ssc.getCriteriaItems()

        self.assertEqual( len( items ), 1 )
        self.assertEqual( len( items[0] ), 2 )
        self.assertEqual( items[0][0], 'foofield' )
        self.assertEqual( items[0][1], 'bar' )


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(SimpleStringCriterionTests),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Permissions used throughout CMFTopic.

$Id: permissions.py 36457 2004-08-12 15:07:44Z jens $
"""
from AccessControl import ModuleSecurityInfo

security = ModuleSecurityInfo('Products.CMFTopic.permissions')

from Products.CMFCore.permissions import setDefaultRoles

security.declarePublic('AddTopics')
AddTopics = 'Add portal topics'
setDefaultRoles(AddTopics, ('Manager',))

security.declarePublic('ChangeTopics')
ChangeTopics = 'Change portal topics'
setDefaultRoles(ChangeTopics, ('Manager', 'Owner',))

security.declarePublic('AccessContentsInformation')
from Products.CMFCore.permissions import AccessContentsInformation

security.declarePublic('ListFolderContents')
from Products.CMFCore.permissions import ListFolderContents

security.declarePublic('View')
from Products.CMFCore.permissions import View



##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" List Criterion: A criterion that is a list

$Id: ListCriterion.py 94489 2009-01-03 16:28:19Z yuppie $
"""

from AccessControl.SecurityInfo import ClassSecurityInfo
from App.class_init import InitializeClass
from zope.interface import implements

from Products.CMFTopic.AbstractCriterion import AbstractCriterion
from Products.CMFTopic.interfaces import ICriterion
from Products.CMFTopic.permissions import ChangeTopics
from Products.CMFTopic.permissions import View
from Products.CMFTopic.Topic import Topic


class ListCriterion( AbstractCriterion ):

    """
        Represent a criterion which is a list of values (for an
        'OR' search).
    """

    implements(ICriterion)

    meta_type = 'List Criterion'
    operator = None
    value = ( '', )

    security = ClassSecurityInfo()

    _editableAttributes = ( 'value', 'operator' )

    def __init__( self, id, field ):
        self.id = id
        self.field = field
        self._clear()

    security.declarePrivate( '_clear' )
    def _clear( self ):
        """
            Restore to original value.
        """
        self.value = ( '', )    # *Not* '()', which won't do at all!
        self.operator = None

    security.declareProtected( ChangeTopics, 'getEditForm' )
    def getEditForm( self ):
        """
            Return the name of skin method which renders the form
            used to edit this kind of criterion.
        """
        return "listc_edit"

    security.declareProtected( ChangeTopics, 'edit' )
    def edit( self, value=None, operator=None ):
        """
            Update the value we match against.
        """
        if value is None:
            self._clear()
        else:
            if type( value ) == type( '' ):
                value = value.split('\n')
            self.value = tuple( value )

        if not operator:
            operator = None

        self.operator = operator

    security.declareProtected(View, 'getCriteriaItems')
    def getCriteriaItems( self ):
        """
            Return a tuple of query elements to be passed to the catalog
            (used by 'Topic.buildQuery()').
        """
        # filter out empty strings
        result = []

        value = tuple( filter( None, self.value ) )
        if not value:
            return ()
        result.append( ( self.field, self.value ), )

        if self.operator is not None:
            result.append( ( '%s_operator' % self.field, self.operator ) )

        return tuple( result )

InitializeClass( ListCriterion )

# Register as a criteria type with the Topic class
Topic._criteriaTypes.append( ListCriterion )
